import _, { isObject } from "lodash"
import { Logger } from "winston"

import { baseErrorCard, basePendingCard, baseSuccessCard } from "./component"

type FunctionMap = Record<
  string,
  { xName: string; xAuthor: string; xIcon: string }
>

type CardMap = Record<string, any>

type TempMap = Record<string, string>

class LarkCard<
  TCardMap extends CardMap,
  TTempMap extends TempMap,
  TFunctionMap extends FunctionMap,
> {
  protected requestId: string
  protected funcName: string
  protected functionMap: TFunctionMap
  protected cardMap: TCardMap
  protected tempMap: TTempMap
  protected stringify: boolean
  protected logger: Logger

  constructor({
    funcName,
    stringify = false,
    requestId,
    cardMap = {} as TCardMap,
    tempMap = {} as TTempMap,
    functionMap = {} as TFunctionMap,
    logger,
  }: {
    funcName: string
    stringify: boolean
    requestId: string
    cardMap?: TCardMap
    tempMap?: TTempMap
    functionMap?: TFunctionMap
    logger: Logger
  }) {
    this.stringify = stringify
    this.requestId = requestId
    this.funcName = funcName
    this.functionMap = functionMap
    this.cardMap = {
      success: baseSuccessCard,
      error: baseErrorCard,
      pending: basePendingCard,
      ...cardMap,
    }
    this.tempMap = tempMap
    this.logger = logger
  }

  setFunction(funcName: string) {
    this.funcName = funcName
  }

  setStringify(stringify: boolean) {
    this.stringify = stringify
  }

  child(func: string, stringify?: boolean) {
    return new LarkCard({
      funcName: func,
      stringify: stringify || this.stringify,
      requestId: this.requestId,
      cardMap: this.cardMap,
      tempMap: this.tempMap,
      functionMap: this.functionMap,
      logger: this.logger,
    })
  }

  genCard(cardKey: keyof TCardMap | object, variables: { [key: string]: any }) {
    const card = isObject(cardKey) ? cardKey : this.cardMap[cardKey]
    if (!card) {
      this.logger.error("卡片未找到", {
        cardKey: String(cardKey),
      })
      throw new Error(`Card ${String(cardKey)} not found`)
    }
    const finalVariables: Record<string, any> = {
      ...variables,
      requestId: this.requestId,
      ...this.functionMap[this.funcName],
    }
    this.logger.debug("卡片最终变量", {
      cardKey: String(cardKey),
      variables: finalVariables,
    })

    const replaceVariables = (str: string): string => {
      const variablePattern = /\$\{(\w+)\}/g
      const matches = str.match(variablePattern)

      if (matches && matches.length === 1 && matches[0] === str) {
        const variableName = matches[0].slice(2, -1)
        return finalVariables[variableName] || ""
      } else {
        return str.replace(variablePattern, (_, v) => finalVariables[v] || "")
      }
    }

    const traverseAndReplace = (obj: any): any => {
      if (_.isString(obj)) {
        return replaceVariables(obj)
      } else if (_.isArray(obj)) {
        return obj.map(traverseAndReplace)
      } else if (_.isObject(obj)) {
        return _.mapValues(obj, traverseAndReplace)
      }
      return obj
    }

    const content = traverseAndReplace(card)

    // 递归遍历content，遇到key是"value"的Object，将variables注入
    const injectVariables = (obj: { value: any }): any => {
      if (_.isObject(obj) && !_.isArray(obj)) {
        if (_.has(obj, "value") && _.isObject(obj.value)) {
          return {
            ...obj,
            value: { ...obj.value, ...finalVariables },
          }
        }
        return _.mapValues(obj, injectVariables)
      } else if (_.isArray(obj)) {
        return obj.map(injectVariables)
      }
      return obj
    }

    const finalContent = injectVariables(content)

    this.logger.debug("卡片最终内容", {
      cardKey: String(cardKey),
      content: finalContent,
    })
    return this.stringify ? JSON.stringify(finalContent) : finalContent
  }

  genSuccessCard(content: any) {
    return this.genCard("success", { content })
  }

  genErrorCard(content: any) {
    return this.genCard("error", { content })
  }

  genPendingCard(content: any) {
    return this.genCard("pending", { content })
  }

  genTempCard(tempKey: keyof TTempMap, variables: { [key: string]: any }) {
    const tempId = this.tempMap[tempKey]
    if (!tempId) {
      this.logger.error("模板未找到", {
        tempKey: String(tempKey),
      })
      throw new Error(`Temp ${String(tempKey)} not found`)
    }
    const finalVariables: Record<string, any> = {
      ...variables,
      requestId: this.requestId,
      ...this.functionMap[this.funcName],
    }
    this.logger.debug("模板最终变量", {
      tempKey: String(tempKey),
      variables: finalVariables,
    })
    const content = {
      type: "template",
      data: {
        config: {
          update_multi: true,
          enable_forward: false,
        },
        template_id: tempId,
        template_variable: finalVariables,
      },
    }
    this.logger.debug("模板最终内容", {
      tempKey: String(tempKey),
      content,
    })
    return this.stringify ? JSON.stringify(content) : content
  }
}

export default LarkCard
